/**
 * @file index.ts
 * @description Barrel export for the technique/intent visualization components.
 * @module components/TechniqueIntent
 *
 * @copyright NVIDIA Corporation 2023-2026
 * @license Apache-2.0
 */

export { default as TechniqueIntentPanel } from "./TechniqueIntentPanel";
export type { TechniqueIntentPanelProps } from "./TechniqueIntentPanel";
export { default as TaxonomyAxisList } from "./TaxonomyAxisList";
