/**
 * @file Theme.ts
 * @description Type definitions for application theming.
 * @module types
 *
 * @copyright NVIDIA Corporation 2023-2026
 * @license Apache-2.0
 */

/** Valid theme values for the application */
export type ThemeValue = "light" | "dark" | "system";

